unit filewtch;
{ TGFGLWatchFilesForDelete beta version .1 copyright Greg Lorriman 1998
D2 and D3 component. Probably works with with D4.

Disclaimer : see at bottom.
}
interface

uses classes;

type TFilesRemovedEvent=procedure(sender : Tobject; files : TStringlist) of object;

type
{Purpose : To allow individual files to be watched for deletion (or move).

Internally uses low priority threads and fileexists function. Passes info on which files have disappeared via event.
This component cannot tell if a file has been renamed rather than deleted
or moved so file name comparison will be needed when updating dependent controls (example below).

To start watching files pass a stringlist of fully qualified file names to the WatchFiles function
having already assigned an event handler to the FilesRemoved event property. To be notified when all the files
have been removed assign a handler to the AllFilesGone event.

See the rest of the class for specific help

Typical usage :

Your FilesRemoved event might look like this :

procedure TMainForm.WatchFilesForDelete1FilesRemoved(Sender : TObject; Files : TStringlist);
var cntr : integer;
begin
	for cntr:=0 to files.count-1 do begin
   	//test whether the filename is still the same as the TListItem with which it was associated (in case of rename)
   	if comparetext(files[cntr],	TFile(TListItem(files.objects[cntr]).data).filename)=0 then
	      	TListItem(files.objects[cntr]).delete;
   end;
end;

TFile is a user defined class like :  Type TFile=class public filename : string; end;
}
	TGFGLWatchFilesForDelete=class(Tcomponent)

   private

   killingthreads : boolean;
   threads : TList;

//Active caused problems so has been disabled
//   FActive : Boolean;
   FHelp : TStrings;
   FStuff : string;
   FVersion : string;
   FFilesRemoved : TFilesRemovedEvent;
   FAllFilesGone : TNotifyEvent;

   protected

   procedure OnthreadTerminating(sender : Tobject);
   procedure OnFilesRemoved(sender : Tobject; files : TStringlist);
   procedure SetActive(ActiveArg : Boolean);
   procedure dummyWrite( s : string);
   procedure setHelp ( h : TStrings);

   public

   constructor create(AOwner: TComponent);override;
   destructor destroy;override;

   {Pass this function the list of files to watch. A low priority thread is
   immediatly created that watches the files for deletion. You should have created
   the 'FilesRemoved' event handler before using this function (in case you are creating this
   component at runtime). Use objects property of the stringlist to carry extra info through to the
   FilesRemoved event. This function can be called multiple times, creating an additional thread for each
   call.}
   procedure WatchFiles(files : TStringlist);

   {The file lists are cleared and the threads destroyed by this function.}
   procedure clear;

   published

   property Help : TStrings  read FHelp  write SetHelp;
   property Version : string read FVersion write dummywrite;
   property OtherGreatStuff : string read FStuff write dummywrite;

   {Files that have been found to be missing are passed as a TStringlist via this event. They are
   removed from the watch list so this event will not be called twice with the same filenames unless you
   add the same filenames in a second call to watchFiles.
   The objects of the file list passed by the WatchFiles method are preserved. Missing files may
   have been renamed rather than moved or deleted; so you may need to carry out filename comparisons
   to establish whether or not to remove dependent control items.}
   property FilesRemoved : TFilesRemovedEvent read FFilesRemoved Write FFilesRemoved;

	{When all the filelists are empty this event is fired}
	property AllFilesGone : TNotifyEvent read FAllFilesGone Write FAllFilesGone ;


end;

procedure Register;

implementation

uses forms,sysutils,windows;

{
 The following class works by watching the files passed by the files property.
 Then it calls the event assigned to the event property to inform
 an outside object of files that have been removed. After which it
 removes the missing files from the watch list. If the watch list
 reduces to 0 it terminates (and is automatically destroyed - look at the
 constructor).

 Typically it is used like this :

 	wft:=TwatchFilesThread.create(true);
   wft.files.assign(files);
   wft.FilesRemoved:=onFilesRemoved;
   wft.Onterminate:=threadTerminating;
   threads.add(wft);
   wft.resume;

   with external code keeping tabs on things with a collection containing
   current threads and via an Onterminate event to remove terminating threads from the
   collection.

   Objects can be assigned to the object[] property of the file list.
}
type
  TWatchFilesThread = class(TThread)
  private
    { Private declarations }
   FFilesRemoved : TFilesRemovedEvent;
   FFiles : TStrings;
   removedFiles : TStringlist;
   error : boolean;

  protected

   procedure Execute; override;
	procedure SetFiles(FilesArg : TStrings);
   procedure signalchange;

	public

	constructor Create(CreateSuspended: Boolean);virtual;
   destructor Destroy;override;

   procedure reset;

	property Files : TStrings read FFiles Write SetFiles ;
   property FilesRemoved : TFilesRemovedEvent read FFilesRemoved Write FFilesRemoved;

end;

constructor TGFGLWatchFilesForDelete.create(AOwner: TComponent);
begin
	inherited create(aOwner);
	threads:=TList.create;
//   active:=true;
   FVersion:='beta .1';
   FStuff :='http://www.lorriman.demon.co.uk';
   FHelp:=TStringlist.create;
   FHelp.add('Usage : submit files via "WatchFiles", reset via "Clear",'+#10+#13+'pause via "active", assign handler to "FilesRemoved"');
   FHelp.add('');
   fhelp.add('Methods :'+#10+#13+' procedure WatchFiles(files : TStringlist); '+#10+#13+' procedure clear;');
end;

destructor TGFGLWatchFilesForDelete.destroy;
begin
	clear;
   fhelp.free;
   threads.free;
   inherited destroy;
end;


procedure TGFGLWatchFilesForDelete.OnthreadTerminating(sender : Tobject);
begin
	if killingthreads then
   	exit;
   threads.delete(threads.indexof(sender));
   threads.pack;
   if assigned(FAllFilesgone) then
	   if threads.count=0 then
   		FallFilesgone(self);
end;

procedure TGFGLWatchFilesForDelete.OnFilesRemoved(sender : Tobject; files : TStringlist);
var
	cntr : integer;
begin
	if application.terminated then
       exit;
   if killingthreads then
   	exit;
{   if not Factive then
   	exit;}

	if assigned(FFilesRemoved) then
   	ffilesremoved(self,files);
end;

procedure TGFGLWatchFilesForDelete.WatchFiles(files : TStringlist);
var
	wft : TwatchFilesThread;
begin
	if files.count=0 then
   	exit;
	wft:=TwatchFilesThread.create(true);
   wft.files.assign(files);
   wft.FilesRemoved:=onFilesRemoved;
   wft.Onterminate:=onThreadTerminating;
   threads.add(wft);
//   if Factive then
	   wft.resume;
end;

procedure TGFGLWatchFilesForDelete.clear;
var
	cntr : integer;
   wasNotActive : boolean;
begin
	if killingthreads then
   	exit;
	try
   	killingthreads:=true;
{       wasNotActive:=Factive;
       if not active then
       	active:=true;
       Factive:=wasNotActive;}
    	for cntr:=0 to threads.count-1 do
       	Tthread(threads[cntr]).terminate;
   finally
   	killingthreads:=false;
   end;
 	threads.clear;
end;


procedure TGFGLWatchFilesForDelete.SetActive(ActiveArg : Boolean);
var
	cntr : integer;
begin
{	if activeArg=Factive then
   	exit;
   Factive:=activeArg;}
//two loops for speed purposes
	if activearg then
   	for cntr:=0 to threads.count-1 do
       	TThread(threads[cntr]).suspended:=false
   else
   	for cntr:=0 to threads.count-1 do
       	TThread(threads[cntr]).suspended:=true;
end;

procedure TGFGLWatchFilesForDelete.dummyWrite( s : string);
begin
end;


{ TWatchFilesThread }

procedure TWatchFilesThread.Execute;
var aFile,cntr : integer;
begin
	try
	while true do begin

       for cntr:=0 to ffiles.count-1 do begin
       	if not fileexists(ffiles[cntr]) then
           	removedfiles.addobject(ffiles[cntr],ffiles.objects[cntr]);
       end;

       if removedfiles.count>0 then begin
           for cntr:=ffiles.count-1 downto 0 do begin
               afile:=removedfiles.indexof(FFiles[cntr]);
               if aFile<>-1 then
                   ffiles.delete(cntr);
           end;
       	synchronize(signalchange);
           removedfiles.clear;
       end;

       if error then
       	exit;

       if ffiles.count=0 then
       	exit;

   	if terminated then
       	exit;
   end;
   except
   end;

end;

procedure TWatchFilesThread.SetFiles(FilesArg : TStrings);
begin
	FFiles.assign(filesarg);
end;

constructor TWatchFilesThread.Create(CreateSuspended: Boolean);
begin
	inherited create(CreateSuspended);
   priority:=tpLower;
   freeonterminate:=true;
	FFiles:=TStringlist.create;
	removedFiles:=TStringlist.create;
end;

destructor TWatchFilesThread.Destroy;
begin
	if assigned(ffiles) then
   	ffiles.free;
	if assigned(filesRemoved) then
   	removedFiles.free;

   inherited destroy;
end;

procedure TWatchFilesThread.signalchange;
begin
   try
	if assigned(FFilesRemoved) then
   	FFilesRemoved(self, removedFiles);
   except
   	on e: exception do begin
   		error:=true;
           application.messagebox(pchar(e.message),'Exception',mb_Ok);
       end;
   end;
end;

procedure TWatchFilesThread.reset;
begin
	ffiles.clear;
   removedfiles.clear;
end;

procedure Register;
begin
  RegisterComponents('Win95', [TGFGLWatchFilesForDelete]);
end;

procedure TGFGLWatchFilesForDelete.setHelp ( h : TStrings);
begin
end;

end.

{DISCLAIMER : This component, and associated files, comes with no
warranties express or implied, and the author does not accept
responsibility for any problems that this component may cause.
Use at your own risk. }
